# latex_generator/latex_generator.py

import os
import base64
from PIL import Image
from io import BytesIO

def generate_latex_table(data):
    # ... (your existing code)

def generate_latex_with_image(data, image_path):
    # ... (your existing code)

if __name__ == "__main__":
    # ... (your existing code)
